%{
/*MA****************************************************************/
/*                                                                 */
/*  FILE     tmod.y                                                */
/*  MODULE   tmod                                                  */
/*  PROGRAM  TMod                                                  */
/*  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           */
/*                                                                 */
/*ME****************************************************************/

#include <iostream>
using std::cerr;

#include "tree.h"

extern int  yylineno;
extern char *yytext;

void yyerror(char *text);
int yylex( void );

%}

%union {
  Tree   *tree;
  Trees  *trees;
  char   *name;
}

%token <name>      NAME NOPARSE
%type  <tree>      TREE
%type  <trees>     TREES

%%

ALL:        ALL TREE                { $2->process(); delete $2; }
          | ALL NOPARSE             { printf("%s\n",$2); free($2); }
          | {}
          ;

TREE:       '(' NAME TREES ')'      { $$=new Tree($2,$3); }
          | NAME                    { $$=new Tree($1); }
          | '(' NAME ')'            { $$=new Tree($2); }
          ;

TREES:      TREES TREE              { $1->push_back($2); $$=$1; }
          | TREE                    { $$ = new Trees; $$->push_back($1); }
          ;

%%


extern FILE  *yyin;

/*FA****************************************************************/
/*                                                                 */
/*  yyerror                                                        */
/*                                                                 */
/*FE****************************************************************/

void yyerror(char *text)

{
  cerr << yylineno << ": " << text << " at: ";
  cerr << yytext << "\naborted.\n";
  exit(1);
}


/*FA****************************************************************/
/*                                                                 */
/*  usage                                                          */
/*                                                                 */
/*FE****************************************************************/

void usage()

{
  fprintf(stderr,"\nUsage: tmod [options] infile\n");
  fprintf(stderr,"\nOPTIONS:\n");
  fprintf(stderr,"-p\tpretty print mode\n");
  fprintf(stderr,"-q\tquiet mode\n");
  fprintf(stderr,"\n");
  exit(0);
}


/*FA****************************************************************/
/*                                                                 */
/*  get_flags                                                      */
/*                                                                 */
/*FE****************************************************************/

void get_flags( int *argc, char **argv )

{
  for( int i=1; i<*argc; i++ ) {
    if (strcmp(argv[i],"-q") == 0) {
      Verbose = false;
      argv[i] = NULL;
    }
    else if (strcmp(argv[i],"-p") == 0) {
      PrettyPrint = true;
      argv[i] = NULL;
    }
    else if (strcmp(argv[i],"-h") == 0)
      usage();
  }
  // remove flags from the argument list
  int k;
  for( int i=k=1; i<*argc; i++)
    if (argv[i] != NULL)
      argv[k++] = argv[i];
  *argc = k;
}


/*FA****************************************************************/
/*                                                                 */
/*  main                                                           */
/*                                                                 */
/*FE****************************************************************/

int main( int argc, char *argv[] )

{
  FILE *file;

  get_flags(&argc, argv);

  if (argc > 1) {
    if ((file = fopen(argv[1],"rt")) == NULL) {
      fprintf(stderr,"\nError: Cannot open input file \"%s\"\n\n", argv[1]);
      exit(1);
    }
  }
  else
    file = stdin;

  if (argc > 2) {
    if ((outfile = fopen(argv[2],"wt")) == NULL) {
      fprintf(stderr,"\nError: Cannot output output file \"%s\"\n\n",argv[2]);
      exit(1);
    }
  }
  else
    outfile = stdout;

  yyin = file;
  try {
    yyparse();
    fclose(file);
  }
  catch(const char* p) {
    cerr << "\n" << p << "\n\n";
    exit(1);
  }

  if (Verbose)
    fputs(" finished\n", stderr);
}
